@@ -138,6 +138,25 @@ is inserted before the cursor, the short name is expanded to its address."
 (defun mew-complete-folder ()
   "Folder complete function."
   (interactive)
+  (if mew-input-folder-search-direction
+      (mew-input-folder-search-complete)
+    (mew-complete-folder2)))
+
+(defun mew-input-folder-search-complete ()
+  (let (keys)
+    (save-excursion
+      (set-buffer mew-input-folder-search-buf)
+      (save-excursion
+	(goto-char (point-min))
+	(while (search-forward (or mew-input-folder-search-key "\n") nil t)
+	  (setq keys
+		(cons (buffer-substring (progn (beginning-of-line) (point))
+					(progn (end-of-line) (point)))
+		      keys)))))
+    (mew-complete-window-show (nreverse (delete "" keys)))
+    (mew-highlight-folder-comp-search-window)))
+
+(defun mew-complete-folder2 ()
   (when (mew-draft-or-header-p)
     (setq completion-ignore-case mew-complete-folder-ignore-case))
   (let ((word (mew-delete-backward-char nil ", \t\n"))
