@@ -19,6 +19,10 @@ requires PTY.")
 (defvar mew-connection-type2 t
   "Connection type for processes that requires a password. ")
 
+(defvar mew-connection-type-for-scan t
+  "Connection type for 'mewls' whose output speed is very fast. 
+To slow down, PTY should be used.")
+
 ;;;
 ;;; Must be here for key setup
 ;;;
