@@ -124,6 +124,8 @@
   (define-key mew-summary-mode-map "ma"   'mew-summary-mark-all)
   (define-key mew-summary-mode-map "mu"   'mew-summary-mark-undo-all)
   (define-key mew-summary-mode-map "mI"   'mew-summary-mark-retrieve-message)
+  (define-key mew-summary-mode-map "ml"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "mlc"  'mew-summary-mark-local-copy)
   (define-key mew-summary-mode-map "V"    'mew-summary-virtual)
   (define-key mew-summary-mode-map "/"    'mew-summary-virtual)
   (define-key mew-summary-mode-map "?"    'mew-summary-search-mark)
@@ -481,15 +483,15 @@
 	(if (boundp 'minibuffer-local-map)
 	    (copy-keymap minibuffer-local-map)
 	  (make-sparse-keymap)))
-  (define-key mew-input-map "\r"     'mew-exit-minibuffer)
-  (define-key mew-input-map "\n"     'mew-exit-minibuffer)
-  (define-key mew-input-map [return] 'mew-exit-minibuffer)
+  (define-key mew-input-map "\r"     'mew-input-exit-minibuffer)
+  (define-key mew-input-map "\n"     'mew-input-exit-minibuffer)
+  (define-key mew-input-map [return] 'mew-input-exit-minibuffer)
+  (define-key mew-input-map ","      'mew-input-comma)
   (define-key mew-input-map " "      'mew-input-complete)
   (define-key mew-input-map "\t"     'mew-input-complete)
   (define-key mew-input-map [tab]    'mew-input-complete)
   (define-key mew-input-map "?"      'mew-input-complete)
-  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-case)
-  (define-key mew-input-map ","      'mew-input-comma))
+  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-case))
 
 (unless mew-input-folder-map
   (setq mew-input-folder-map (copy-keymap mew-input-map))
@@ -498,12 +500,12 @@
       (define-key mew-input-folder-map (char-to-string c)
 	'mew-input-folder-self-insert)
       (setq c (1+ c))))
-  (define-key mew-input-folder-map "\C-g"   'mew-abort-minibuffer2)
-  (define-key mew-input-folder-map "\r"     'mew-exit-minibuffer2)
-  (define-key mew-input-folder-map "\n"     'mew-exit-minibuffer2)
-  (define-key mew-input-folder-map [return] 'mew-exit-minibuffer2)
-  (define-key mew-input-folder-map [delete] 'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map [delete]    'mew-input-folder-self-insert)
   (define-key mew-input-folder-map [backspace] 'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map "\C-g"   'mew-input-folder-abort-minibuffer)
+  (define-key mew-input-folder-map "\r"     'mew-input-folder-exit-minibuffer)
+  (define-key mew-input-folder-map "\n"     'mew-input-folder-exit-minibuffer)
+  (define-key mew-input-folder-map [return] 'mew-input-folder-exit-minibuffer)
   (define-key mew-input-folder-map "+"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "%"      'mew-input-folder-prefix)
   (define-key mew-input-folder-map "$"      'mew-input-folder-prefix)
