@@ -266,7 +266,7 @@ according to 'mew-local-folder<'. Binary search is used for speed reasons."
   ;; 'scan folder range
   ;; 'vir  opts count func
   (mew-summary-with-mewls
-   (let* ((process-connection-type mew-connection-type2)
+   (let* ((process-connection-type mew-connection-type-for-scan)
 	  (bnm (mew-summary-folder-name 'ext))
 	  (pnm (mew-local-info-name bnm))
 	  (buf (get-buffer-create (mew-local-buffer-name bnm)))
