@@ -125,10 +125,10 @@
   (when (> end begin)
     (save-excursion
       (mew-elet
-       ;; Need to inherit properties in a cache.
-       ;; So, we use buffer-substring, not mew-buffer-substring.
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep properties (e.g. citation color)
+       ;; in a message cache.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (when mew-insert-final-newline
 	 (save-excursion
 	   (goto-char (point-max))
@@ -144,8 +144,10 @@
     (set-buffer (mew-buffer-message))
     (mew-elet
      (mew-erase-buffer)
-     (insert
-      (save-excursion (set-buffer cache) (buffer-substring begin end)))
+     ;; We need to keep properties (e.g. citation color)
+     ;; in a message cache.
+     ;; This must be "insert-buffer-substring".
+     (insert-buffer-substring cache begin end)
      (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
 		  (or (mew-cache-dinfo-get-decode-broken cache)
 		      (mew-charset-to-cs
@@ -172,7 +174,9 @@
     (save-excursion
       (let ((start (point)))
 	(mew-elet
-	 (mew-insert-buffer-substring cache begin end)
+	 ;; We need to keep composite properties of charset.
+	 ;; This must be "insert-buffer-substring".
+	 (insert-buffer-substring cache begin end)
 	 ;; Highlight
 	 (when mew-use-text/enriched
 	   (condition-case nil
@@ -196,9 +200,9 @@
       (save-excursion
 	(mew-elet
 	 (let ((start (point)))
-	   (insert
-	    (save-excursion
-	      (set-buffer cache) (mew-buffer-substring begin end)))
+	   ;; We need to keep composite properties of charset.
+	   ;; This must be "insert-buffer-substring".
+	   (insert-buffer-substring cache begin end)
 	   (funcall mew-prog-text/html start (point-max)))))
     (mew-elet
      (insert " #     # ####### #     # #\n"
@@ -225,8 +229,9 @@
     (save-excursion
       (mew-elet
        (mew-erase-buffer)
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep composite properties of charset.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
 		    (or (mew-cache-dinfo-get-decode-broken cache)
 			(mew-charset-to-cs
@@ -295,9 +300,9 @@
       (save-excursion
 	(mew-elet
 	 (let ((start (point)))
-	   (insert
-	    (save-excursion
-	      (set-buffer cache) (mew-buffer-substring begin end)))
+	   ;; We need to keep composite properties of charset.
+	   ;; This must be "insert-buffer-substring".
+	   (insert-buffer-substring cache begin end)
 	   (funcall mew-prog-text/xml start (point-max)))))
     (mew-elet
      (insert " #     # #     # #\n"
@@ -324,8 +329,9 @@
     (save-excursion
       (mew-elet
        (mew-erase-buffer)
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep composite properties of charset.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (unless (and mew-inherit-ct (mew-ct-textp mew-inherit-ct)
 		    (or (mew-cache-dinfo-get-decode-broken cache)
 			(mew-charset-to-cs
@@ -388,9 +394,9 @@
       (save-excursion
 	(mew-elet
 	 (let ((start (point)))
-	   (insert
-	    (save-excursion
-	      (set-buffer cache) (mew-buffer-substring begin end)))
+	   ;; We need to keep composite properties of charset.
+	   ;; This must be "insert-buffer-substring".
+	   (insert-buffer-substring cache begin end)
 	   (funcall mew-prog-application/xml start (point-max)))))
     (mew-elet
      (insert " #     # #     # #\n"
@@ -417,8 +423,9 @@
     (save-excursion
       (mew-elet
        (mew-erase-buffer)
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep composite properties of charset.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (funcall mew-prog-application/xml-ext (point-min) (point-max))
        (mew-summary-display-postscript 'nohook)))
     (message "Displaying XML...done"))
@@ -490,10 +497,9 @@
   (when (> end begin)
     (save-excursion
       (mew-elet
-       ;; Need to inherit properties in a cache.
-       ;; So, we use buffer-substring, not mew-buffer-substring.
-       (insert
-	(save-excursion (set-buffer cache) (buffer-substring begin end)))
+       ;; We need to keep properties of a header.
+       ;; This must be "insert-buffer-substring".
+       (insert-buffer-substring cache begin end)
        (mew-header-arrange (point-min) (point-max))))))
 
 (defun mew-mime-message/rfc822 (cache part)
@@ -502,6 +508,7 @@
    (let* ((hbeg (mew-syntax-get-begin part))
 	  (hend (mew-syntax-get-end   part))
 	  (body (mew-syntax-get-part part)))
+     ;; We need to keep properties of a header.
      ;; This must be "insert-buffer-substring".
      (insert-buffer-substring cache hbeg hend)
      (mew-header-arrange (point-min) (point-max))
