@@ -12,7 +12,9 @@
 ;;; Completion switch
 ;;;
 
-(defvar mew-input-complete-function nil)
+(defvar mew-input-complete-function nil
+  "A function to be called when TAB is typed in minibuffer.
+This is used in 'mew-input-complete'.")
 
 (defun mew-input-complete ()
   "Do completion according to the global variable
@@ -21,17 +23,21 @@
   (if (and mew-input-complete-function (fboundp mew-input-complete-function))
       (funcall mew-input-complete-function)))
 
-(defvar mew-exit-minibuffer-function nil)
+(defvar mew-input-exit-minibuffer-function nil
+  "A function to be called when RET is typed in minibuffer.
+This function are used to check validity of 'case' and sort key.")
 
-(defun mew-exit-minibuffer ()
+(defun mew-input-exit-minibuffer ()
   "Ensure the input meets a condition."
   (interactive)
-  (if (or (not (and mew-exit-minibuffer-function
-                    (fboundp mew-exit-minibuffer-function)))
-          (funcall mew-exit-minibuffer-function))
+  (if (or (not (and mew-input-exit-minibuffer-function
+                    (fboundp mew-input-exit-minibuffer-function)))
+          (funcall mew-input-exit-minibuffer-function))
       (exit-minibuffer)))
-
-(defvar mew-input-comma-function nil)
+ 
+(defvar mew-input-comma-function nil
+  "A function to be called when ',' is typed in minibuffer.
+This function can be used to check validity of 'case'.")
 
 (defun mew-input-comma ()
   "Ensure the input meets a condition."
@@ -43,7 +49,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Mew original completion
+;;; Clean up side effects
 ;;;
 
 (defun mew-input-clear ()
@@ -60,6 +66,8 @@ at completions."
 ;;; Folder
 ;;;
 
+(defvar mew-input-folder-search-buf " *Mew* folder search")
+
 (defvar mew-input-folder-hist nil)
 (defvar mew-input-folder-search-direction nil)
 (defvar mew-input-folder-search-key nil)
@@ -68,38 +76,221 @@ at completions."
 (defvar mew-input-folder-search-point nil)
 (defvar mew-input-folder-search-multi nil)
 
-(defun mew-input-folder-prefix ()
-  (interactive)
-  (if mew-input-folder-search-direction
-      (mew-input-folder-self-insert)
-    (let ((pos (- (point) (mew-minibuf-point-min))))
-      (cond
-       ((= pos 0)
-	(insert last-command-char))
-       ((and (= pos 1)
-	     (save-excursion
-	       (forward-char -1)
-	       (looking-at "[-+%$]"))) ;; excluding *
-	(forward-char -1)
-	(delete-char 1)
-	(insert last-command-char))
-       ((and (> pos 1)
-	     (save-excursion
-	       (forward-char -2)
-	       (looking-at "[:,][-+*%$]")))
-	(forward-char -1)
-	(delete-char 1)
-	(insert last-command-char))
-       (t
-	(insert last-command-char))))))
+(defun mew-input-folder-clean-up ()
+  (setq mew-input-folder-search-direction nil)
+  (setq mew-input-folder-search-key nil)
+  (setq mew-input-folder-search-match nil)
+  (setq mew-input-folder-search-original nil)
+  (setq mew-input-folder-search-point nil))
+
+(defun mew-highlight-folder-comp-search-window ()
+  (let* ((win (get-buffer-window mew-buffer-completions))
+	 (match mew-input-folder-search-match)
+	 regex face)
+    (when (and win
+	       mew-input-folder-search-key
+	       mew-input-folder-search-match)
+      (save-excursion
+	(set-buffer mew-buffer-completions)
+	(mew-elet
+	 (goto-char (point-min))
+	 (when (looking-at "^Key: ")
+	   (delete-region (point-min) (progn (forward-line) (point))))
+	 (insert "Key: " mew-input-folder-search-key ",  "
+		 "Match: " mew-input-folder-search-match "\n")
+	 (put-text-property (point-min) (point-max) 'face nil)
+	 (setq regex (format "\\(%s\\)" (regexp-quote match)))
+	 (when (or (re-search-forward (concat regex "[ \t]") nil t)
+		   (re-search-forward (concat regex "$") nil t))
+	   (goto-char (match-beginning 1))
+	   (setq face (if (not window-system)
+			  'mew-face-header-from
+			(if (or (and mew-xemacs-p (find-face 'isearch))
+				(and (not mew-xemacs-p) (facep 'isearch)))
+			    'isearch
+			  'region)))
+	   (put-text-property (point) (match-end 1) 'face face)
+	   (unless (pos-visible-in-window-p (point) win)
+	     (set-window-start win (progn (forward-line -2) (point))))))))))
 
 (defun mew-input-folder-display (&optional msg)
-  (delete-region (mew-minibuf-point-min) (point-max))
-  (insert "(" (or mew-input-folder-search-match "") ") ")
-  (insert (or mew-input-folder-search-key ""))
-  (if msg (save-excursion (insert " [" msg "]"))))
+  (mew-highlight-folder-comp-search-window)
+  (mew-elet
+   (delete-region (mew-minibuf-point-min) (point-max))
+   (insert "(" (or mew-input-folder-search-match "") ") ")
+   (insert (or mew-input-folder-search-key ""))
+   (if msg (save-excursion (insert " [" msg "]")))
+   (put-text-property (mew-minibuf-point-min) (point-max) 'read-only t)))
+
+(defun mew-input-folder-search-setup (&optional min)
+  (let* (case:fld case fld alist buf)
+    (save-excursion
+      (goto-char (point-max))
+      (if (search-backward "," nil t)
+	  (setq mew-input-folder-search-point
+		(- (match-end 0) (mew-minibuf-point-min)))))
+    (setq mew-input-folder-search-original
+	  (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+    (setq case:fld mew-input-folder-search-original)
+    (if mew-input-folder-search-point
+	(setq case:fld (substring case:fld mew-input-folder-search-point)))
+    (setq case (car (mew-folder-case case:fld)))
+    (setq fld (mew-folder-folder case:fld))
+    (mew-input-folder-display)
+    (if (or (null fld) (string= fld ""))
+	(setq fld (mew-proto case)))
+    (setq buf (get-buffer-create mew-input-folder-search-buf))
+    (cond
+     ((mew-folder-popp fld)
+      (setq alist (mew-pop-folder-alist)))
+     ((mew-folder-nntpp fld)
+      (setq alist (mew-nntp-folder-alist case)))
+     ((mew-folder-imapp fld)
+      (setq alist (mew-imap-folder-alist case)))
+     (t
+      (setq alist (mew-local-folder-alist))))
+    (save-excursion
+      (set-buffer buf)
+      (setq buffer-read-only t)
+      (mew-elet
+       (mew-erase-buffer)
+       (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
+      (if min (goto-char (point-min))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Folder search forward
+;;;
+
+(defun mew-input-folder-search-forward-1 (&optional again)
+  (let (no-match pos)
+    (save-excursion
+      (set-buffer mew-input-folder-search-buf)
+      (setq pos (point))
+      (if again
+	  (forward-line)
+	(beginning-of-line))
+      (if (search-forward mew-input-folder-search-key nil t)
+	  (progn
+	    (goto-char (match-beginning 0))
+	    (setq mew-input-folder-search-match
+		  (mew-buffer-substring
+		   (save-excursion (beginning-of-line) (point))
+		   (save-excursion (end-of-line) (point)))))
+	(setq no-match t)
+	(goto-char pos)))
+    (if no-match 
+	(mew-input-folder-display "no match")
+      (mew-input-folder-display))))
+
+(defun mew-input-folder-search-forward ()
+  "Search a folder forward."
+  (interactive)
+  (cond
+   ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
+    (mew-input-folder-display "no match"))
+   ((eq mew-input-folder-search-direction 'forward)
+    (mew-input-folder-search-forward-1 'again))
+   ((eq mew-input-folder-search-direction 'backward)
+    (setq mew-input-folder-search-direction 'forward)
+    (mew-input-folder-search-forward-1 'again))
+   (t
+    (setq mew-input-folder-search-direction 'forward)
+    (mew-input-folder-search-setup 'min))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Folder search backward
+;;;
+
+(defun mew-input-folder-search-backward-1 (&optional again)
+  (let (no-match pos)
+    (save-excursion
+      (set-buffer mew-input-folder-search-buf)
+      (setq pos (point))
+      (if again
+	  (progn (forward-line -1) (end-of-line))
+	(end-of-line))
+      (if (search-backward mew-input-folder-search-key nil t)
+	  (progn
+	    (goto-char (match-end 0))
+	    (setq mew-input-folder-search-match
+		  (mew-buffer-substring
+		   (save-excursion (beginning-of-line) (point))
+		   (save-excursion (end-of-line) (point)))))
+	(setq no-match t)
+	(goto-char pos)))
+    (if no-match 
+	(mew-input-folder-display "no match")
+      (mew-input-folder-display))))
+
+(defun mew-input-folder-search-backward ()
+  "Search a folder backward."
+  (interactive)
+  (cond
+   ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
+    (mew-input-folder-display "no match"))
+   ((eq mew-input-folder-search-direction 'forward)
+    (setq mew-input-folder-search-direction 'backward)
+    (mew-input-folder-search-backward-1 'again))
+   ((eq mew-input-folder-search-direction 'backward)
+    (mew-input-folder-search-backward-1 'again))
+   (t
+    (setq mew-input-folder-search-direction 'backward)
+    (mew-input-folder-search-setup))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Folder commands
+;;;
+
+(defun mew-input-folder-exit-minibuffer ()
+  (interactive)
+  (if mew-input-folder-search-direction
+      (mew-elet
+       (delete-region (mew-minibuf-point-min) (point-max))
+       (cond
+	(mew-input-folder-search-match
+	 (when mew-input-folder-search-point
+	   (insert (substring mew-input-folder-search-original
+			      0 mew-input-folder-search-point)))
+	 (insert mew-input-folder-search-match))
+	(mew-input-folder-search-original
+	 (insert mew-input-folder-search-original)))
+       (mew-input-folder-clean-up)
+       (mew-complete-window-delete))
+    (exit-minibuffer)))
+
+(defun mew-input-folder-abort-minibuffer ()
+  "This function exits folder search mode if in folder search mode.
+Otherwise, it exits minibuffer."
+  (interactive)
+  (if mew-input-folder-search-direction
+      (mew-elet
+       (delete-region (mew-minibuf-point-min) (point-max))
+       (when mew-input-folder-search-original
+	 (insert mew-input-folder-search-original))
+       (mew-input-folder-clean-up)
+       (mew-complete-window-delete))
+    (abort-recursive-edit)))
+
+(defun mew-input-folder-comma ()
+  "This function inserts ',' in a single folder input
+and does not insert anything in a multiple folder input."
+  (interactive)
+  (if (and (null mew-input-folder-search-direction)
+	   mew-input-folder-search-multi)
+      (insert ",")))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Self insert
+;;;
 
 (defun mew-input-folder-self-insert ()
+  "This function normally inserts its bound key to minibuffer.
+When in folder search mode, this function searches a candidate
+folder and displays it in addition to its bound key."
   (interactive)
   (let* ((key (if mew-xemacs-p
 		  (event-key last-command-event)
@@ -119,8 +310,12 @@ at completions."
       (setq gfunc (lookup-key (current-global-map) key))
       (unless gfunc
 	(setq key (lookup-key function-key-map key))
-	(if (vectorp key)
-	    (setq gfunc (lookup-key (current-global-map) key))))))
+	(cond
+	 ((vectorp key) ;; normal Emacs
+	  (setq gfunc (lookup-key (current-global-map) key)))
+	 ((stringp key) ;; Meadow
+	  (setq last-str key)
+	  (setq gfunc (lookup-key (current-global-map) key)))))))
     (if mew-input-folder-search-direction
 	(cond
 	 ((or (equal key "\177") (equal key [127])
@@ -158,169 +353,56 @@ at completions."
        ((and (fboundp gfunc) (commandp gfunc))
 	(call-interactively gfunc))))))
 
-(defvar mew-input-folder-search-buf "*Mew folder search") ;; xxx
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Prefix hack
+;;;
 
-(defun mew-input-folder-search-forward-1 (&optional again)
-  (let (no-match)
-    (save-excursion
-      (set-buffer mew-input-folder-search-buf)
-      (if again
-	  (forward-char (length mew-input-folder-search-key))
-	(beginning-of-line))
-      (if (search-forward mew-input-folder-search-key nil t)
-	  (progn
-	    (goto-char (match-beginning 0))
-	    (setq mew-input-folder-search-match
-		  (mew-buffer-substring
-		   (save-excursion (beginning-of-line) (point))
-		   (save-excursion (end-of-line) (point)))))
-	(setq no-match t)
-	(if again
-	    (forward-char (- (length mew-input-folder-search-key)))
-	  (setq mew-input-folder-search-key
-		(substring mew-input-folder-search-key 0 -1)))))
-    (if no-match 
-	(mew-input-folder-display "no match")
-      (mew-input-folder-display))))
-
-(defun mew-input-folder-search-backward-1 (&optional again)
-  (let (no-match)
-    (save-excursion
-      (set-buffer mew-input-folder-search-buf)
-      (if again
-	  (forward-char (- (length mew-input-folder-search-key)))
-	(end-of-line))
-      (if (search-backward mew-input-folder-search-key nil t)
-	  (progn
-	    (goto-char (match-end 0))
-	    (setq mew-input-folder-search-match
-		  (mew-buffer-substring
-		   (save-excursion (beginning-of-line) (point))
-		   (save-excursion (end-of-line) (point)))))
-	(setq no-match t)
-	(if again
-	    (forward-char (length mew-input-folder-search-key))
-	  (setq mew-input-folder-search-key
-		(substring mew-input-folder-search-key 0 -1)))))
-    (if no-match 
-	(mew-input-folder-display "no match")
-      (mew-input-folder-display))))
-
-(defun mew-input-folder-search-forward ()
-  (interactive)
-  (cond
-   ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
-    (mew-input-folder-display "no match"))
-   ((eq mew-input-folder-search-direction 'forward)
-    (mew-input-folder-search-forward-1 'again))
-   ((eq mew-input-folder-search-direction 'backward)
-    (setq mew-input-folder-search-direction 'forward)
-    (mew-input-folder-search-forward-1 'again))
-   (t
-    (setq mew-input-folder-search-direction 'forward)
-    (mew-input-folder-search-setup 'min))))
-
-(defun mew-input-folder-search-backward ()
-  (interactive)
-  (cond
-   ((and mew-input-folder-search-direction (null mew-input-folder-search-key))
-    (mew-input-folder-display "no match"))
-   ((eq mew-input-folder-search-direction 'forward)
-    (setq mew-input-folder-search-direction 'backward)
-    (mew-input-folder-search-backward-1 'again))
-   ((eq mew-input-folder-search-direction 'backward)
-    (mew-input-folder-search-backward-1 'again))
-   (t
-    (setq mew-input-folder-search-direction 'backward)
-    (mew-input-folder-search-setup))))
-
-(defun mew-input-folder-search-setup (&optional min)
-  (let* (case:fld case fld alist buf)
-    (save-excursion
-      (goto-char (point-max))
-      (if (search-backward "," nil t)
-	  (setq mew-input-folder-search-point
-		(- (match-end 0) (mew-minibuf-point-min)))))
-    (setq mew-input-folder-search-original
-	  (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
-    (setq case:fld mew-input-folder-search-original)
-    (if mew-input-folder-search-point
-	(setq case:fld (substring case:fld mew-input-folder-search-point)))
-    (setq case (car (mew-folder-case case:fld)))
-    (setq fld (mew-folder-folder case:fld))
-    (mew-input-folder-display)
-    (if (or (null fld) (string= fld ""))
-	(setq fld (mew-proto case)))
-    (setq buf (get-buffer-create mew-input-folder-search-buf))
-    (cond
-     ((mew-folder-popp fld)
-      (setq alist (mew-pop-folder-alist)))
-     ((mew-folder-nntpp fld)
-      (setq alist (mew-nntp-folder-alist case)))
-     ((mew-folder-imapp fld)
-      (setq alist (mew-imap-folder-alist case)))
-     (t
-      (setq alist (mew-local-folder-alist))))
-    (save-excursion
-      (set-buffer buf)
-      (setq buffer-read-only t)
-      (mew-elet
-       (mew-erase-buffer)
-       (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
-      (if min (goto-char (point-min))))))
-
-(defun mew-exit-minibuffer2 ()
+(defun mew-input-folder-prefix ()
+  "A function to insert a folder prefix in minibuffer.
+If the previous character is another folder prefix, 
+it is deleted automatically."
   (interactive)
   (if mew-input-folder-search-direction
-      (progn
-	(delete-region (mew-minibuf-point-min) (point-max))
-	(cond
-	 (mew-input-folder-search-match
-	  (when mew-input-folder-search-point
-	    (insert (substring mew-input-folder-search-original
-			       0 mew-input-folder-search-point)))
-	  (insert mew-input-folder-search-match))
-	 (mew-input-folder-search-original
-	  (insert mew-input-folder-search-original)))
-	(mew-input-folder-clean-up))
-    (exit-minibuffer)))
+      (mew-input-folder-self-insert)
+    (let ((pos (- (point) (mew-minibuf-point-min))))
+      (cond
+       ((= pos 0)
+	(insert last-command-char))
+       ((and (= pos 1)
+	     (save-excursion
+	       (forward-char -1)
+	       (looking-at "[-+%$]"))) ;; excluding *
+	(forward-char -1)
+	(delete-char 1)
+	(insert last-command-char))
+       ((and (> pos 1)
+	     (save-excursion
+	       (forward-char -2)
+	       (looking-at "[:,][-+*%$]")))
+	(forward-char -1)
+	(delete-char 1)
+	(insert last-command-char))
+       (t
+	(insert last-command-char))))))
 
-(defun mew-abort-minibuffer2 ()
-  (interactive)
-  (if mew-input-folder-search-direction
-      (progn
-	(delete-region (mew-minibuf-point-min) (point-max))
-	(when mew-input-folder-search-original
-	  (insert mew-input-folder-search-original))
-	(mew-input-folder-clean-up))
-    (abort-recursive-edit)))
-
-(defun mew-input-folder-comma ()
-  (interactive)
-  (if (and (null mew-input-folder-search-direction)
-	   mew-input-folder-search-multi)
-      (insert ",")))
-
-(defun mew-input-folder-clean-up ()
-  (setq mew-input-folder-search-direction nil)
-  (setq mew-input-folder-search-key nil)
-  (setq mew-input-folder-search-match nil)
-  (setq mew-input-folder-search-original nil)
-  (setq mew-input-folder-search-point nil))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Folder
+;;;
 
 (defun mew-input-folder (default &optional local-only)
   "Input a folder from the minibuffer.
 DEFAULT must be a folder WITHOUT case.
 This function returns a folder."
+  (mew-input-clear)
+  (mew-input-folder-clean-up)
+  (setq mew-input-folder-search-multi nil)
   (let ((prefix (mew-folder-prefix default))
 	(mew-inherit-case (mew-sinfo-get-case))
+	(mew-input-complete-function
+	 (if local-only 'mew-complete-local-folder 'mew-complete-folder))
 	folder)
-    (mew-input-clear)
-    (mew-input-folder-clean-up)
-    (setq mew-input-folder-search-multi nil)
-    (if local-only
-	(setq mew-input-complete-function 'mew-complete-local-folder)
-      (setq mew-input-complete-function 'mew-complete-folder))
     (setq folder (read-from-minibuffer (format "Folder name (%s): " default)
 				       prefix mew-input-folder-map nil
 				       'mew-input-folder-hist))
@@ -331,7 +413,15 @@ This function returns a folder."
 
 (defun mew-input-folders (default &optional proto)
   ;; Returns a list
-  (let ((mew-inherit-case (mew-sinfo-get-case))
+  (mew-input-clear)
+  (mew-input-folder-clean-up)
+  (setq mew-input-folder-search-multi t)
+  (let ((mew-input-complete-function (if proto
+					 (if (mew-folder-imapp proto)
+					     'mew-complete-imap-folder
+					   'mew-complete-local-folder)
+				       'mew-complete-folder))
+	(mew-inherit-case (mew-sinfo-get-case))
 	prompt init folders protos)
     (cond
      ;; Must be without case.
@@ -341,14 +431,6 @@ This function returns a folder."
      (t
       (setq prompt "Folder name: ")
       (setq init (mew-join "," default))))
-    (mew-input-clear)
-    (mew-input-folder-clean-up)
-    (setq mew-input-folder-search-multi t)
-    (if proto
-	(if (mew-folder-imapp proto)
-	    (setq mew-input-complete-function 'mew-complete-imap-folder)
-	  (setq mew-input-complete-function 'mew-complete-local-folder))
-      (setq mew-input-complete-function 'mew-complete-folder))
     (setq folders (read-from-minibuffer prompt
 					init mew-input-folder-map nil
 					'mew-input-folder-hist))
@@ -374,8 +456,8 @@ This function returns a folder."
 
 (defun mew-input-address (prompt &optional default)
   (mew-input-clear)
-  (setq mew-input-complete-function 'mew-complete-address)
-  (let (val vals addrs ret)
+  (let ((mew-input-complete-function 'mew-complete-address)
+	val vals addrs ret)
     (setq val (read-from-minibuffer 
 	       (if default (format prompt default) prompt)
 	       ""
@@ -402,8 +484,8 @@ This function returns a folder."
 
 (defun mew-input-pick-pattern ()
   (mew-input-clear)
-  (setq mew-input-complete-function 'mew-complete-pick-pattern)
-  (let ((keymap (copy-keymap mew-input-map)))
+  (let ((mew-input-complete-function 'mew-complete-pick-pattern)
+	(keymap (copy-keymap mew-input-map)))
     (define-key keymap " " nil)
     (mew-pick-macro-expand-string
      (read-from-minibuffer "Pick pattern: "
@@ -428,7 +510,7 @@ This function returns a folder."
 
 (defvar mew-input-sort-key-hist nil)
 
-(defun mew-sort-key-check ()
+(defun mew-input-sort-key-check ()
   (let* ((field:mode (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
 	 (mode (car (cdr (mew-split field:mode ?:))))
 	 err)
@@ -443,8 +525,8 @@ This function returns a folder."
 
 (defun mew-input-sort-key (key)
   (mew-input-clear)
-  (setq mew-input-complete-function 'mew-complete-sort-key)
-  (let* ((mew-exit-minibuffer-function 'mew-sort-key-check)
+  (let* ((mew-input-complete-function 'mew-complete-sort-key)
+	 (mew-input-exit-minibuffer-function 'mew-input-sort-key-check)
 	 (field:mode (read-from-minibuffer
 		      (format "Sort by (%s)? : " key)
 		      ""
@@ -469,13 +551,13 @@ This function returns a folder."
 
 (defun mew-input-rfile (prompt) ;; prompt="To:"
   (mew-input-clear)
-  (setq mew-input-complete-function 'mew-complete-rfile)
-  (read-from-minibuffer
-   (concat prompt " ")
-   ""
-   mew-input-map
-   nil
-   'mew-input-rfile-hist))
+  (let ((mew-input-complete-function 'mew-complete-rfile))
+    (read-from-minibuffer
+     (concat prompt " ")
+     ""
+     mew-input-map
+     nil
+     'mew-input-rfile-hist)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -671,7 +753,7 @@ This function returns a folder."
 ;;; Case
 ;;;
 
-(defun mew-case-check ()
+(defun mew-input-case-check ()
   (let* ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
          (lst (mew-split case ?,))
 	 err)
@@ -687,21 +769,21 @@ This function returns a folder."
       t)))
 
 (defun mew-input-case (default msg &optional edit)
-  (let (case lst ret)
-    (unless default (setq default mew-case-default))
-    (mew-input-clear)
-    (setq mew-input-complete-function 'mew-complete-case)
-    (let ((mew-exit-minibuffer-function 'mew-case-check)
-          (mew-input-comma-function 'mew-case-check))
-      (if edit
-	  (setq case (read-from-minibuffer
-		      (format "%s case value: " msg)
-		      default
-		      mew-input-map))
+  (mew-input-clear)
+  (unless default (setq default mew-case-default))
+  (let ((mew-input-complete-function 'mew-complete-case)
+	(mew-input-exit-minibuffer-function 'mew-input-case-check)
+	(mew-input-comma-function 'mew-input-case-check)
+	case lst ret)
+    (if edit
 	(setq case (read-from-minibuffer
-		    (format "%s case value (%s): " msg default)
-		    ""
-		    mew-input-map))))
+		    (format "%s case value: " msg)
+		    default
+		    mew-input-map))
+      (setq case (read-from-minibuffer
+		  (format "%s case value (%s): " msg default)
+		  ""
+		  mew-input-map)))
     (if (string= case "")
 	default
       (setq lst (mew-split case ?,))
