@@ -165,7 +165,7 @@
 	  (insert str)
 	  (or (mew-multibyte-string-p str) (mew-set-buffer-multibyte t))
 	  (decode-coding-region (point-min) (point-max) cs)
-	  (setq ret (buffer-substring (point-min) (point-max))))
+	  (setq ret (buffer-substring (point-min) (point-max)))) ;; xxx
 	(mew-remove-buffer buf)
 	ret))))
 
