@@ -13,37 +13,22 @@
 ;;;
 
 (defvar mew-smtp-info-list
-  '("status" "recipients" "orig-recipients"
-    "messages" "error" "lock" "ssh-process" "auth"
-    "string" "cont" "timer" "server" "ssh-server" "port"
-    "raw-header" "from" "bcc" "dcc" "fcc" "logtime" "msgid"
-    "helo-domain" "user" "auth" "auth-list" "queue" "case"
-    "sender"))
+  '(;; parameters to be saved
+    "raw-header" "recipients" "orig-recipients"
+    "bcc" "dcc" "fcc" "msgid" "logtime"
+    ;; parameters used internally
+    "case" "server" "port" "ssh-server"
+    "user" "auth-list"
+    "helo-domain" 
+    "status" "ssh-process" "messages"
+    ;; parameters used internally and should be initialized
+    "string" "error" "auth-selected" "timer" "cont" "from" "sender"
+    "done"))
 
 (mew-info-defun "mew-smtp-" mew-smtp-info-list)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Process name
-;;;
-
-(defconst mew-smtp-info-prefix "mew-smtp-info-")
-
-(defsubst mew-smtp-info-name (case)
-  (let ((server (mew-smtp-server case))
-	(port (mew-smtp-port case))
-	(user (mew-smtp-user case))
-	(auth (mew-smtp-auth case))
-	(sshsrv (mew-smtp-ssh-server case))
-	(name mew-smtp-info-prefix))
-    (if auth
-	(setq name (concat name user "@" server))
-      (setq name (concat name server)))
-    (unless (string= port mew-smtp-port)
-      (setq name (concat name ":" port)))
-    (if sshsrv
-	(concat name "%" sshsrv)
-      name)))
+(defvar mew-smtp-info-list-save-length 8)
+(defvar mew-smtp-info-list-clean-length 18)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -51,43 +36,185 @@
 ;;;
 
 (defvar mew-smtp-fsm
-  '(("greeting"     ("220" . "ehlo"))
-    ("ehlo"         ("250" . "auth") (t . "helo"))
+  '(("greeting"      ("220" . "ehlo"))
+    ("ehlo"          ("250" . "auth") (t . "helo"))
     ;;
-    ("auth"         ("250" . "mail-from"))
-    ("cram-md5"     ("334" . "pwd-cram-md5") (t . "wpwd"))
-    ("pwd-cram-md5" ("235" . "mail-from") (t . "wpwd"))
-    ("login"        ("334" . "pwd-login") (t . "wpwd"))
-    ("pwd-login"    ("235" . "mail-from") (t . "wpwd"))
-    ("plain"        ("235" . "mail-from") (t . "wpwd"))
+    ("auth"          ("250" . "mail-from"))
+    ("auth-cram-md5" ("334" . "pwd-cram-md5") (t . "wpwd"))
+    ("pwd-cram-md5"  ("235" . "mail-from") (t . "wpwd"))
+    ("auth-login"    ("334" . "pwd-login") (t . "wpwd"))
+    ("pwd-login"     ("235" . "mail-from") (t . "wpwd"))
+    ("auth-plain"    ("235" . "mail-from") (t . "wpwd"))
     ;;
-    ("helo"         ("250" . "mail-from"))
-    ("mail-from"    ("250" . "rcpt-to"))
-    ("rcpt-to"      ("250" . "data"))
-    ("data"         ("354" . "content"))
-    ("content"      ("250" . "done"))
-    ("done"         ("250" . "noop"))))
+    ("helo"          ("250" . "mail-from"))
+    ("mail-from"     ("250" . "rcpt-to"))
+    ("rcpt-to"       ("250" . "data"))
+    ("data"          ("354" . "content"))
+    ("content"       ("250" . "done"))
+    ("quit"          (t     . "noop"))))
 
 (defsubst mew-smtp-fsm-by-status (status)
   (assoc status mew-smtp-fsm))
 
 (defsubst mew-smtp-fsm-next (status code)
-  (cdr (mew-assoc-match code (cdr (mew-smtp-fsm-by-status status)) 0)))
+  (cdr (mew-assoc-match2 code (cdr (mew-smtp-fsm-by-status status)) 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; SMTP AUTH
+;;; Filters
 ;;;
 
-(defun mew-smtp-passtag (pnm auth)
-  (concat (downcase auth)
-	  "/" (mew-smtp-get-user pnm)
-	  "@" (mew-smtp-get-server pnm)
-	  ":" (mew-smtp-get-port pnm)))
+(defun mew-smtp-command-ehlo (pro pnm)
+  (let ((helo-domain (mew-smtp-get-helo-domain pnm)))
+    (mew-smtp-process-send-string pro "EHLO %s" helo-domain)))
 
-(defun mew-smtp-input-passwd (prompt pnm auth)
-  (let ((tag (mew-smtp-passtag pnm auth)))
-    (mew-input-passwd prompt tag)))
+(defun mew-smtp-command-helo (pro pnm)
+  (let ((helo-domain (mew-smtp-get-helo-domain pnm)))
+    (mew-smtp-process-send-string pro "HELO %s" helo-domain)))
+
+(defun mew-smtp-command-auth (pro pnm)
+  (cond
+   ((mew-smtp-get-user pnm)
+    (let ((auth-list (mew-smtp-get-auth-list pnm))
+	  (str (mew-smtp-get-string pnm)) auth func)
+      (if (and (string-match "AUTH \\([^\n\r]+\\)\r?\n" str)
+	       (setq auth (mew-auth-select (match-string 1 str) auth-list))
+	       (setq func (mew-smtp-auth-get-func auth))
+	       (fboundp func))
+	  (progn
+	    (mew-smtp-set-auth-selected pnm auth)
+	    (funcall func pro pnm))
+	(mew-smtp-debug "<AUTH>" "No preferred SMTP AUTH.\n")
+	;; xxx how should we act?
+	(mew-smtp-set-status pnm "mail-from")
+	(mew-smtp-command-mail-from pro pnm))))
+   (t
+    (mew-smtp-set-status pnm "mail-from")
+    (mew-smtp-command-mail-from pro pnm))))
+
+(defun mew-smtp-command-wpwd (pro pnm)
+  (let ((auth (mew-smtp-get-auth-selected pnm)))
+    (mew-passwd-set-passwd (mew-smtp-passtag pnm auth) nil)
+    (mew-smtp-recover pro pnm (format "Wrong password for %s" auth))))
+
+(defun mew-smtp-command-mail-from (pro pnm)
+  (widen)
+  (clear-visited-file-modtime)
+  ;;
+  (let* ((case (mew-smtp-get-case pnm))
+	 (resentp (mew-header-existp mew-resent-from:))
+	 (sender: (if resentp mew-resent-sender: mew-sender:))
+	 (from: (if resentp mew-resent-from: mew-from:))
+	 (from (mew-header-get-value from:))
+	 (froms (mew-addrstr-parse-address-list from))
+	 (nfrom (length froms))
+	 (mail-from (mew-smtp-mail-from case)))
+    (mew-smtp-set-from pnm from) ;; for Bcc:
+    (unless mail-from
+      ;; Which address is suitable for MAIL FROM if multiple?
+      (setq mail-from (car froms)))
+    (unless (mew-header-existp sender:)
+      (if (= nfrom 1)
+	  (if (and mew-use-sender (not (string= mail-from (car froms))))
+	      (mew-smtp-set-sender pnm (cons sender: mail-from)))
+	(mew-smtp-set-sender pnm (cons sender: mail-from))))
+    ;;
+    (mew-smtp-process-send-string pro "MAIL FROM:<%s>" mail-from)))
+
+(defun mew-smtp-command-rcpt-to (pro pnm)
+  (let* ((recipients (mew-smtp-get-recipients pnm))
+	 (recipient (car recipients)))
+    (setq recipients (cdr recipients))
+    (mew-smtp-set-recipients pnm recipients)
+    (if recipients (mew-smtp-set-status pnm "mail-from"))
+    (mew-smtp-process-send-string pro "RCPT TO:<%s>" recipient)))
+
+(defun mew-smtp-command-data (pro pnm)
+  (goto-char (point-max))
+  (unless (bolp) (insert "\n"))
+  (mew-dot-insert)
+  (mew-eol-fix-for-write)
+  (set-buffer-modified-p nil)
+  (mew-smtp-set-cont pnm (point-min))
+  (mew-smtp-set-timer pnm nil)
+  (mew-smtp-process-send-string pro "DATA"))
+
+(defun mew-smtp-command-content (pro pnm)
+  (save-excursion
+    (let ((cont (mew-smtp-get-cont pnm))
+	  (sender (mew-smtp-get-sender pnm))
+          (inc 1000) (i 0) (N 10))
+      (set-buffer (process-buffer pro))
+      ;; Sender:
+      (when sender
+	(mew-smtp-process-send-string pro "%s %s" (car sender) (cdr sender))
+	(mew-smtp-set-sender pnm nil))
+      ;;
+      (while (and (< cont (point-max)) (not (input-pending-p)) (< i N))
+        (let ((next (min (point-max) (+ cont inc))))
+          (process-send-region pro cont next) ;; xxx
+          (setq cont next)
+          (setq i (1+ i))))
+      (mew-smtp-set-cont pnm cont)
+      (if (< cont (point-max))
+          (let ((timer
+                 (if (input-pending-p)
+                     (run-with-idle-timer
+		      0.01 nil 'mew-smtp-command-content pro pnm)
+                   (run-at-time 0.05 nil 'mew-smtp-command-content pro pnm))))
+            (mew-smtp-set-timer pnm timer))
+        (mew-smtp-set-cont pnm nil)
+        (mew-smtp-set-timer pnm nil)
+        (mew-smtp-process-send-string pro ".")))))
+
+(defun mew-smtp-command-done (pro pnm)
+  (let ((fcc (mew-smtp-get-fcc pnm))
+	(back (mew-queue-backup (buffer-file-name) mew-queue-info-suffix))
+	(buf (process-buffer pro))
+	msgs)
+    ;; mew-folder-new-message may be slow if the folder contains
+    ;; a lot of messages. So, let's Fcc in background.
+    (mew-net-fcc-message fcc back)
+    (mew-smtp-log pnm)
+    (if (mew-smtp-get-bcc pnm)
+	(mew-smtp-bcc pro pnm back)
+      ;; killing buffer
+      (set-process-buffer pro nil)
+      (mew-remove-buffer buf)
+      (setq msgs (mew-smtp-get-messages pnm))
+      (if (and msgs
+	       (mew-queue-get-next
+		pnm msgs
+		mew-smtp-info-list-save-length
+		'mew-smtp-set-messages))
+	  ;; A file inserted. Now in msg's buffer
+	  (progn
+	    (mew-info-clean-up pnm mew-smtp-info-list-clean-length)
+	    (set-process-buffer pro (current-buffer))
+	    (mew-smtp-set-status pnm "mail-from")
+	    (mew-smtp-command-mail-from pro pnm))
+	(mew-smtp-set-status pnm "quit")
+	(mew-smtp-command-quit pro pnm)))))
+
+(defun mew-smtp-command-quit (pro pnm)
+  (mew-smtp-set-done pnm t)
+  (mew-smtp-process-send-string pro "QUIT"))
+
+(defun mew-smtp-command-noop (pro pnm)
+  ())
+
+(defun mew-smtp-recover (pro pnm err)
+  (let ((case (mew-smtp-get-case pnm)))
+    (mew-smtp-log pnm err)
+    (mew-smtp-queue case err)
+    (when (and (processp pro) (eq (process-status pro) 'open))
+      (mew-smtp-set-status pnm "quit")
+      (mew-smtp-command-quit pro pnm))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; AUTH
+;;;
 
 (defvar mew-smtp-auth-alist
   '(("CRAM-MD5" mew-smtp-command-auth-cram-md5)
@@ -100,129 +227,72 @@
 ;;
 
 (defun mew-smtp-command-auth-cram-md5 (pro pnm)
-  (process-send-string
-   pro (format "AUTH CRAM-MD5%s" mew-cs-eol))
-  (mew-smtp-set-status pnm "cram-md5"))
+  (mew-smtp-process-send-string pro "AUTH CRAM-MD5")
+  (mew-smtp-set-status pnm "auth-cram-md5"))
 
 (defun mew-smtp-command-pwd-cram-md5 (pro pnm)
   (let ((str (mew-smtp-get-string pnm))
 	(user (mew-smtp-get-user pnm))
 	challenge passwd cram-md5)
-    (if (string-match " \\([A-Za-z0-9+/]+=*\\)" str) ;; xxx
-	(setq challenge (mew-match 1 str)))
+    (if (string-match " \\([a-zA-Z0-9+/]+=*\\)" str) ;; xxx
+	(setq challenge (match-string 1 str)))
     (setq passwd (mew-smtp-input-passwd "CRAM-MD5 password: " pnm "cram-md5"))
     (setq cram-md5 (mew-cram-md5 user passwd challenge))
-    (process-send-string pro (format "%s%s" cram-md5 mew-cs-eol))))
+    (mew-smtp-process-send-string pro cram-md5)))
 
 (defun mew-smtp-command-auth-login (pro pnm)
   (let* ((user (mew-smtp-get-user pnm))
 	 (euser (mew-base64-encode-string user)))
-    (process-send-string pro (format "AUTH LOGIN %s%s" euser mew-cs-eol))
-    (mew-smtp-set-status pnm "login")))
+    (mew-smtp-process-send-string pro "AUTH LOGIN %s" euser)
+    (mew-smtp-set-status pnm "auth-login")))
 
 (defun mew-smtp-command-pwd-login (pro pnm)
   (let* ((passwd (mew-smtp-input-passwd "LOGIN password: " pnm "login"))
-	 (login (mew-base64-encode-string passwd)))
-    (process-send-string pro (format "%s%s" login mew-cs-eol))))
+	 (epasswd (mew-base64-encode-string passwd)))
+    (mew-smtp-process-send-string pro epasswd)))
 
 (defun mew-smtp-command-auth-plain (pro pnm)
   (let* ((passwd (mew-smtp-input-passwd "PLAIN password: " pnm "plain"))
 	 (user (mew-smtp-get-user pnm))
 	 (plain (mew-base64-encode-string (format "\0%s\0%s" user passwd))))
-    (process-send-string pro (format "AUTH PLAIN %s%s" plain mew-cs-eol))
-    (mew-smtp-set-status pnm "plain")))
+    (mew-smtp-process-send-string pro "AUTH PLAIN %s" plain)
+    (mew-smtp-set-status pnm "auth-plain")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Commands
+;;; Sub functions
 ;;;
 
-(defun mew-smtp-send-message (pnm &optional unlock)
-  (let* ((process (get-process pnm))
-	 (server (mew-smtp-get-server pnm))
-	 (port (mew-smtp-get-port pnm))
-	 (ssh-server (mew-smtp-get-ssh-server pnm))
-	 (ret t)
-	 sshpro sshname errmsg opened lport)
-    (cond
-     ((null (mew-smtp-get-recipients pnm))
-      (setq ret nil)
-      (message "No recipient!"))
-     ((and (processp process) (not unlock) (mew-smtp-get-lock pnm))
-      (setq errmsg "SMTP connection is locked")
-      (mew-smtp-set-error pnm errmsg)
-      (mew-smtp-queue pnm errmsg))
-     (t
-      (if (and (processp process)
-	       (memq (process-status process) '(open)))
-	  (setq opened t))
-      (mew-smtp-set-string pnm nil)
-      (cond
-       (opened
-	(set-process-buffer process (current-buffer))
-	(set-process-sentinel process 'mew-smtp-sentinel)
-	(set-process-filter process 'mew-smtp-filter)
-	(mew-smtp-command-mail-from process pnm)
-	(mew-smtp-set-status pnm "mail-from")
-	(message "Sending in background..."))
-       (t
-	(if (null ssh-server)
-	    (setq process (mew-smtp-open pnm server port))
-	  (setq sshpro (mew-open-ssh-stream server port ssh-server))
-	  (mew-smtp-set-ssh-process pnm sshpro)
-	  (if (null sshpro)
-	      (setq errmsg (concat "cannot connect to " ssh-server))
-	    (setq sshname (process-name sshpro))
-	    (setq lport (mew-ssh-pnm-to-lport sshname))
-	    (if lport (setq process (mew-smtp-open pnm "localhost" lport)))))
-	(cond
-	 (process
-	  (mew-smtp-set-status pnm "greeting")
-	  (set-process-buffer process (current-buffer))
-	  (set-process-sentinel process 'mew-smtp-sentinel)
-	  (set-process-filter process 'mew-smtp-filter)
-	  (message "Sending in background..."))
-	 (t
-	  (mew-smtp-queue pnm errmsg)
-	  (mew-smtp-tear-down pnm)))))))
-    ret))
+(defconst mew-smtp-info-prefix "mew-smtp-info-")
 
-(defun mew-smtp-flush-queue (qfld case)
-  (let* ((server (mew-smtp-server case))
-	 (ssh-server (mew-smtp-ssh-server case))
-	 (pnm (mew-smtp-info-name case))
-	 msgs flushp-msgs flushp)
-    (cond
-     ((mew-smtp-get-messages pnm)
-      (message "%s is being flushed" qfld))
-     ((mew-smtp-get-lock pnm)
-      (message "SMTP connection for %s is locked" qfld))
-     (t
-      ;; cannot lock here
-      (mew-summary-clean-folder-cache qfld)
-      (setq msgs (directory-files (mew-expand-folder qfld)
-				  'full mew-regex-message-files))
-      (setq flushp-msgs (mew-smtp-get-next pnm msgs)) ;; a file inserted
-      (setq flushp (car flushp-msgs))
-      (mew-smtp-set-case pnm case)
-      (mew-smtp-set-server pnm server)
-      (mew-smtp-set-port pnm (mew-smtp-port case))
-      (mew-smtp-set-ssh-server pnm ssh-server)
-      (mew-smtp-set-queue pnm qfld)
-      (mew-smtp-set-messages pnm (cdr flushp-msgs))
-      (mew-smtp-set-auth pnm (mew-smtp-auth case))
-      (mew-smtp-set-auth-list pnm (mew-smtp-auth-list case))
-      (mew-smtp-set-user pnm (mew-smtp-user case))
-      (mew-smtp-set-helo-domain pnm (mew-smtp-helo-domain case))
-      (mew-smtp-set-sender pnm nil)
-      ;; If Mail-From is specified for the case, override the old one.
-      (if (mew-smtp-mail-from case)
-	  (mew-smtp-set-from pnm (mew-smtp-mail-from case)))
-      ;; in msg's buffer
-      (when flushp
-	(message "Flushing %s..." qfld)
-	(mew-smtp-send-message pnm))))
-    (run-hooks 'mew-smtp-flush-hook))) ;; xxx
+(defsubst mew-smtp-info-name (case)
+  (let ((server (mew-smtp-server case))
+	(port (mew-smtp-port case))
+	(user (mew-smtp-user case))
+	(sshsrv (mew-smtp-ssh-server case))
+	(name mew-smtp-info-prefix))
+    (if user
+	(setq name (concat name user "@" server))
+      (setq name (concat name server)))
+    (unless (string= port mew-smtp-port)
+      (setq name (concat name ":" port)))
+    (if sshsrv
+	(concat name "%" sshsrv)
+      name)))
+
+(defun mew-smtp-process-send-string (pro &rest args)
+  (let ((str (apply 'format args)))
+    (process-send-string pro (concat str mew-cs-eol))))
+
+(defun mew-smtp-passtag (pnm auth)
+  (concat (downcase auth)
+	  "/" (mew-smtp-get-user pnm)
+	  "@" (mew-smtp-get-server pnm)
+	  ":" (mew-smtp-get-port pnm)))
+
+(defun mew-smtp-input-passwd (prompt pnm auth)
+  (let ((tag (mew-smtp-passtag pnm auth)))
+    (mew-input-passwd prompt tag)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -231,53 +301,103 @@
 
 (defun mew-smtp-open (pnm server port)
   (let ((sprt (mew-port-sanity-check port))
-	errmsg process tm)
-    (mew-smtp-set-error pnm nil)
+	pro tm errmsg)
     (condition-case emsg
 	(progn
 	  (setq tm (mew-timer mew-smtp-timeout-time 'mew-smtp-timeout))
-	  (message "Connecting to %s with %s port..." server port)
-	  (setq process (open-network-stream pnm nil server sprt))
-	  (process-kill-without-query process)
-	  (mew-set-process-cs process mew-cs-text-for-net mew-cs-text-for-net)
-	  (message "Connecting to %s with %s port...done" server port))
+	  (message "Connecting to the SMTP server...")
+	  (setq pro (open-network-stream pnm nil server sprt))
+	  (process-kill-without-query pro)
+	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
+	  (message "Connecting to the SMTP server...done"))
       (quit
-       (setq errmsg (format "cannot make a %s connection to %s" port server))
+       (setq errmsg "Cannot connect to the SMTP server")
        (mew-smtp-set-error pnm errmsg)
-       (setq process nil)
+       (setq pro nil)
        (message errmsg))
       (error
-       (setq errmsg (nth 1 emsg))
+       (setq errmsg (format "%s, %s" (nth 1 emsg) (nth 2 emsg)))
        (mew-smtp-set-error pnm errmsg)
-       (setq process nil)
+       (setq pro nil)
        (message errmsg)))
     (if tm (cancel-timer tm))
-    process))
+    pro))
 
 (defun mew-smtp-timeout ()
   (signal 'quit nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Closing SMTP
+;;; Launcher
 ;;;
 
-(defun mew-smtp-close (pnm)
-  (interactive)
-  (let ((process (get-process pnm)))
-    (if (and (processp process) (memq (process-status process) '(open)))
-	(progn
-	  (set-process-buffer process nil)
-	  (set-process-filter process nil)
-	  (process-send-string process (format "QUIT%s" mew-cs-eol)))
-      (mew-smtp-tear-down pnm))))
+(defun mew-smtp-send-message (case)
+  (let* ((server (mew-smtp-server case))
+	 (port (mew-smtp-port case))
+	 (pnm (mew-smtp-info-name case))
+	 (sshsrv (mew-smtp-ssh-server case))
+	 (sslp (mew-smtp-ssl case))
+	 (sslport (mew-smtp-ssl-port case))
+	 process sshname sshpro lport errmsg)
+    (cond
+     ((null (mew-smtp-get-recipients pnm))
+      (message "No recipient!"))
+     ((get-process pnm)
+      (setq errmsg "SMTP connection is locked")
+      (mew-smtp-set-error pnm errmsg)
+      (mew-smtp-queue case errmsg))
+     (t
+      (cond
+       (sshsrv
+	(setq sshpro (mew-open-ssh-stream server port sshsrv))
+	(mew-smtp-set-ssh-process pnm sshpro)
+	(when sshpro
+	  (setq sshname (process-name sshpro))
+	  (setq lport (mew-ssh-pnm-to-lport sshname))
+	  (when lport
+	    (setq process (mew-smtp-open pnm "localhost" lport)))))
+       (sslp
+	(setq process (mew-open-ssl-stream pnm nil server sslport)))
+       (t
+	(setq process (mew-smtp-open pnm server port))))
+      (if (null process)
+	  (progn
+	    (setq errmsg "Cannot connect to the SMTP server")
+	    (mew-smtp-set-error pnm errmsg)
+	    (mew-smtp-queue case errmsg))
+	(mew-info-clean-up pnm mew-smtp-info-list-clean-length)
+	(mew-smtp-set-case pnm case)
+	(mew-smtp-set-server pnm server)
+	(mew-smtp-set-port pnm port)
+	(mew-smtp-set-ssh-server pnm sshsrv)
+	(mew-smtp-set-helo-domain pnm (mew-smtp-helo-domain case))
+	(mew-smtp-set-user pnm (mew-smtp-user case))
+	(mew-smtp-set-auth-list pnm (mew-smtp-auth-list case))
+	(mew-smtp-set-status pnm "greeting")
+	(set-process-buffer process (current-buffer))
+	(set-process-sentinel process 'mew-smtp-sentinel)
+	(set-process-filter process 'mew-smtp-filter)
+	(message "Sending in background..."))))))
 
-(defun mew-smtp-tear-down (pnm)
-  (let ((process (get-process pnm))
-	(sshpro (mew-smtp-get-ssh-process pnm)))
-    (if (processp process) (delete-process process))
-    (if (processp sshpro) (delete-process sshpro))
-    (mew-info-clean-up pnm)))
+(defun mew-smtp-flush-queue (case &optional qfld)
+  (let* ((pnm (mew-smtp-info-name case))
+	 msgs)
+    (unless qfld
+      (setq qfld (mew-queue-folder case)))
+    (if (get-process pnm)
+	(message "SMTP connection for %s is locked" qfld)
+      (mew-summary-folder-cache-clean qfld)
+      (setq msgs (directory-files (mew-expand-folder qfld)
+				  'full mew-regex-message-files))
+      (when (and msgs
+		 (mew-queue-get-next
+		  pnm msgs
+		  mew-smtp-info-list-save-length
+		  'mew-smtp-set-messages))
+	;; A file inserted. Now in msg's buffer
+	(run-hooks 'mew-smtp-flush-hook)
+	(message "Flushing %s..." qfld)
+	(mew-smtp-send-message case)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -303,7 +423,7 @@
      (cond
       ((and (string-match "\n$" str)
 	    (string-match "^\\([1-5][0-7][0-9]\\) " str))
-       (setq code (mew-match 1 str))
+       (setq code (match-string 1 str))
        (setq next (mew-smtp-fsm-next status code))
        (cond
 	(next
@@ -312,204 +432,64 @@
 	 (and func (funcall func process pnm))
 	 (mew-smtp-set-string pnm nil))
 	(t
-	 ;; (mew-passwd-set-passwd mew-draft-smtp-server nil) ;; xxx
 	 (if (string-match "^pwd-" status)
 	     (message "Password is wrong"))
-	 (mew-smtp-recover pnm str))))
+	 (mew-smtp-recover process pnm str))))
       (t ()))))) ;; stay
 
 (defun mew-smtp-sentinel (process event)
-  (let ((pnm (process-name process)))
+  (let* ((pnm (process-name process))
+	 (done (mew-smtp-get-done pnm))
+	 (sshpro (mew-smtp-get-ssh-process pnm)))
     (mew-smtp-debug "SMTP SENTINEL" event)
-    (mew-smtp-tear-down pnm)
+    (mew-info-clean-up pnm)
+    (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	(process-send-string sshpro "exit\n"))
+    (if done (message "Sending in background...done"))
     (run-hooks 'mew-smtp-sentinel-hook)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Filters
+;;; Queuing
 ;;;
 
-(defun mew-smtp-command-ehlo (pro pnm)
-  (let ((helo-domain (mew-smtp-get-helo-domain pnm)))
-    (process-send-string pro (format "EHLO %s%s" helo-domain mew-cs-eol))))
-
-(defun mew-smtp-command-helo (pro pnm)
-  (let ((helo-domain (mew-smtp-get-helo-domain pnm)))
-    (process-send-string pro (format "HELO %s%s" helo-domain mew-cs-eol))))
-
-(defun mew-smtp-command-auth (pro pnm)
-  (cond
-   ((mew-smtp-get-auth pnm)
-    (let ((str (mew-smtp-get-string pnm)) auth func)
-      (if (and (string-match "AUTH \\([^\n\r]+\\)\r?\n" str)
-	       (setq auth (mew-auth-select
-			   (mew-match 1 str) mew-smtp-auth-list))
-	       (setq func (mew-smtp-auth-get-func auth))
-	       (fboundp func))
-	  (progn
-	    (mew-smtp-set-auth pnm auth)
-	    (funcall func pro pnm))
-	(mew-smtp-debug "<AUTH>" "No preferred SMTP AUTH.\n")
-	;; xxx how should we act?
-	(mew-smtp-set-status pnm "mail-from")
-	(mew-smtp-command-mail-from pro pnm))))
-   (t
-    (mew-smtp-set-status pnm "mail-from")
-    (mew-smtp-command-mail-from pro pnm))))
-
-(defun mew-smtp-command-wpwd (pro pnm)
-  (let ((auth (mew-smtp-get-auth pnm)))
-    (mew-passwd-set-passwd (mew-smtp-passtag pnm auth) nil)
-    (mew-smtp-recover pnm (format "Wrong password for %s" auth))))
-
-(defun mew-smtp-command-mail-from (pro pnm)
-  ;; Session is about to start. Let's lock the connection first.
-  (let ((from (mew-smtp-get-from pnm)))
-    (mew-smtp-set-lock pnm t)
-    (process-send-string pro (format "MAIL FROM:<%s>%s" from mew-cs-eol))))
-
-(defun mew-smtp-command-rcpt-to (pro pnm)
-  (let* ((recipients (mew-smtp-get-recipients pnm))
-	 (recipient (car recipients)))
-    (setq recipients (cdr recipients))
-    (mew-smtp-set-recipients pnm recipients)
-    (if recipients (mew-smtp-set-status pnm "mail-from"))
-    (process-send-string pro (format "RCPT TO:<%s>%s" recipient mew-cs-eol))))
-
-(defun mew-smtp-command-data (pro pnm)
-  (widen)
-  (clear-visited-file-modtime)
-  ;;
-  (let* ((resentp (mew-header-existp mew-resent-from:))
-	 (sender: (if resentp mew-resent-sender: mew-sender:))
-	 (from: (if resentp mew-resent-from: mew-from:)))
-    (unless (mew-header-existp sender:)
-      (let* ((from (mew-addrstr-parse-address-list 
-		    (mew-header-get-value from:)))
-	     (nfrom (length from))
-	     (mail-from (mew-smtp-get-from pnm)))
-	(if (= nfrom 1)
-	    (if (and mew-use-sender (not (string= mail-from (car from))))
-		(mew-smtp-set-sender pnm (cons sender: mail-from)))
-	  (mew-smtp-set-sender pnm (cons sender: mail-from))))))
-  ;;
-  (goto-char (point-max))
-  (unless (bolp) (insert "\n"))
-  (goto-char (point-min))
-  (while (re-search-forward "^\\." nil t)
-    (insert ".")
-    (forward-line))
-  (when (string= mew-cs-eol "\r\n")
-    (goto-char (point-min))
-    (while (search-forward "\n" nil t) (replace-match "\r\n" nil t)))
-  (set-buffer-modified-p nil)
-  (mew-smtp-set-cont pnm (point-min))
-  (mew-smtp-set-timer pnm nil)
-  (process-send-string pro (format "DATA%s" mew-cs-eol)))
-
-(defun mew-smtp-command-content (pro pnm)
-  (save-excursion
-    (let ((cont (mew-smtp-get-cont pnm))
-	  (sender (mew-smtp-get-sender pnm))
-          (inc 1000) (i 0) (N 10))
-      (set-buffer (process-buffer pro))
-      ;; Sender:
-      (when (and sender (= cont (point-min)))
-	(process-send-string
-	 pro (format "%s %s%s" (car sender) (cdr sender) mew-cs-eol))
-	;; Ensuring cont is not (point-min) in the next time
-        (let ((next (min (point-max) (+ cont inc))))
-          (process-send-region pro cont next)
-          (setq cont next)
-          (setq i (1+ i))))
-      ;;
-      (while (and (< cont (point-max)) (not (input-pending-p)) (< i N))
-        (let ((next (min (point-max) (+ cont inc))))
-          (process-send-region pro cont next)
-          (setq cont next)
-          (setq i (1+ i))))
-      (mew-smtp-set-cont pnm cont)
-      (if (< cont (point-max))
-          (let ((timer
-                 (if (input-pending-p)
-                     (run-with-idle-timer
-		      0.01 nil 'mew-smtp-command-content pro pnm)
-                   (run-at-time 0.05 nil 'mew-smtp-command-content pro pnm))))
-            (mew-smtp-set-timer pnm timer))
-        (mew-smtp-set-cont pnm nil)
-        (mew-smtp-set-timer pnm nil)
-        (process-send-string pro (format ".%s" mew-cs-eol))))))
-
-(defun mew-queue-backup (orig)
-  (let* ((norg (file-name-sans-extension orig))
-	 (back (mew-prepend-prefix norg mew-backup-prefix))
-	 (info (concat norg mew-queue-info-suffix))
-	 (ibck (mew-prepend-prefix info mew-backup-prefix)))
-    (if (file-exists-p orig) (rename-file orig back 'override))
-    (if (file-exists-p info) (rename-file info ibck 'override))
-    back))
-
-(defun mew-smtp-command-done (pro pnm)
-  (let* ((back (mew-queue-backup (buffer-file-name)))
-	 (fcc (mew-smtp-get-fcc pnm))
-	 fcc-file link fld buf)
-    ;; mew-folder-new-message may be slow if the folder contains
-    ;; a lot of messages. So, let's Fcc in background.
-    (catch 'loop
-      (while fcc
-	(setq fld (car fcc))
-	(mew-folder-check fld 'force-to-create)
-	(setq fcc (cdr fcc))
-	(setq fcc-file (mew-folder-new-message fld))
-	(when fcc-file
-	  (copy-file back fcc-file)
-	  (set-file-modes fcc-file mew-file-mode)
-	  (mew-touch-folder fld)
-	  (throw 'loop nil))))
-    (while fcc
-      (setq fld (car fcc))
-      (mew-folder-check fld 'force-to-create)
-      (setq fcc (cdr fcc))
-      (setq link (mew-folder-new-message fld))
-      (when link
-	(mew-link fcc-file link)
-	(mew-touch-folder fld)))
-    (mew-smtp-log pnm)
-    (if (mew-smtp-get-bcc pnm)
-	(mew-smtp-bcc pnm back)
-      ;; killing buffer
-      (setq buf (process-buffer pro))
-      (set-process-buffer pro nil)
-      (mew-remove-buffer buf)
-      (mew-smtp-send-next-message pnm))))
-
-(defun mew-smtp-send-next-message (pnm &optional no-msg)
-  (if (mew-smtp-get-messages pnm)
-      (let* ((msgs (mew-smtp-get-messages pnm))
-	     (flushp-msgs (mew-smtp-get-next pnm msgs)) ;; a file inserted
-	     (flushp (car flushp-msgs)))
-	;; in msg's buffer
-	(mew-smtp-set-messages pnm (cdr flushp-msgs))
-	(if flushp (mew-smtp-send-message pnm 'unlock)))
-    (mew-smtp-set-status pnm "noop")
-    (mew-smtp-set-lock pnm nil)
-    (unless mew-smtp-keep-connection (mew-smtp-close pnm))
-    (or no-msg (message "Sending in background...done"))))
-
-(defun mew-smtp-command-noop (pro pnm)
-  ())
+(defun mew-smtp-queue (case err)
+  ;; Must be in a buffer where a message is contained.
+  (let* ((pnm (mew-smtp-info-name case))
+	 (qfld (mew-queue-folder case))
+	 (oname (buffer-name))
+	 (work (buffer-file-name))
+	 file-info file info nname)
+    (mew-local-folder-check qfld)
+    (setq file-info (mew-queue-enqueue work qfld))
+    (setq file (nth 0 file-info) info (nth 1 file-info))
+    (setq nname (mew-concat-folder qfld (file-name-nondirectory file)))
+    ;;
+    (let* ((n mew-smtp-info-list-save-length)
+	   (data (make-vector n nil))
+	   (i 0))
+      (while (< i n)
+	(aset data i (aref (mew-info pnm) i))
+	(setq i (1+ i)))
+      (mew-smtp-set-recipients data (mew-smtp-get-recipients data))
+      (mew-lisp-save info data))
+    ;;
+    (mew-remove-buffer (current-buffer))
+    (message "%s has been queued to %s (%s)"
+	     oname nname (or (mew-smtp-get-error pnm) err))
+    (mew-touch-folder qfld)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Bcc:
 ;;;
 
-(defun mew-smtp-bcc (pnm back)
+(defun mew-smtp-bcc (pro pnm back)
   (let* ((dir (file-name-directory back))
 	 (msg (file-name-nondirectory back))
 	 (case (mew-smtp-get-case pnm)))
     (mew-elet
-     (erase-buffer)
+     (mew-erase-buffer)
      (mew-set-buffer-multibyte t)
      (mew-smtp-set-recipients pnm (mew-smtp-get-bcc pnm))
      (mew-smtp-set-orig-recipients pnm (mew-smtp-get-bcc pnm))
@@ -520,7 +500,6 @@
      (mew-draft-header-insert mew-to: "Bcc-Receiver:;")
      (mew-draft-header-insert mew-subj: mew-bcc-subject)
      (mew-draft-header-insert mew-from: (mew-smtp-get-from pnm))
-     ;; xxx Sender:
      (mew-draft-header-insert mew-organization: (mew-organization case))
      (mew-draft-header-insert-alist (mew-header-alist case))
      ;; X-Mailer: must be the last
@@ -539,93 +518,10 @@
      (mew-encode-make-header)
      (mew-encode-save-draft)
      (mew-overlay-delete-buffer)
-     (mew-smtp-send-message pnm 'unlock))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Queuing
-;;;
-
-;; qdir -> qfld
-
-(defun mew-smtp-queue (pnm err)
-  ;; Must be in a buffer where a message is contained.
-  (let* ((oname (buffer-name))
-	 (orig (buffer-file-name))
-	 (qfld (mew-smtp-get-queue pnm))
-	 file file-info nname)
-    (mew-folder-check qfld 'force-to-create)
-    (if (and (string-match (concat mew-queue-work-suffix "$") orig)
-	     ;; another process has already queued?
-	     (not (file-exists-p (file-name-sans-extension orig))))
-	;; +queue/1.wrk
-	(setq file (file-name-sans-extension orig))
-      ;; +draft/1
-      (setq file (mew-folder-new-message qfld)))
-    (rename-file orig file 'override)
-    (unless mew-use-nfs-hack
-      ;; This cause an error when using NFS.
-      (set-file-modes file mew-file-mode))
-    (setq file-info (concat file mew-queue-info-suffix))
-    (setq nname (concat (file-name-as-directory qfld)
-			(file-name-nondirectory file)))
-    (mew-lisp-save file-info (mew-info pnm))
-    (mew-remove-buffer (current-buffer))
-    (message "%s has been queued to %s (%s)"
-	     oname nname (or (mew-smtp-get-error pnm) err))
-    (mew-touch-folder qfld)
-    (mew-smtp-set-error pnm nil)))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Next message
-;;;
-
-(defun mew-smtp-insert-file (pnm file)
-  (when (file-readable-p file)
-    (let* ((work (concat file mew-queue-work-suffix))
-	   (info (concat file mew-queue-info-suffix))
-	   (server (mew-smtp-get-server pnm))
-	   (port (mew-smtp-get-port pnm))
-	   (ssh-server (mew-smtp-get-ssh-server pnm)))
-      (rename-file file work 'override)
-      (mew-frwlet
-       mew-cs-text-for-read mew-cs-dummy
-       (set-buffer (find-file-noselect work)))
-      (set (intern pnm) (mew-lisp-load info))
-      (mew-smtp-set-server pnm server)
-      (mew-smtp-set-port pnm port)
-      (mew-smtp-set-ssh-server pnm ssh-server)
-      (mew-smtp-set-recipients pnm (mew-smtp-get-orig-recipients pnm))
-      t)))
-
-(defun mew-smtp-get-next (pnm msgs)
-  (when msgs
-    (let* ((msg (car msgs))
-	   flushp)
-      (setq msgs (cdr msgs))
-      (if (mew-smtp-insert-file pnm msg) (setq flushp t))
-      (catch 'loop
-	(while (not flushp)
-	  (setq msg (car msgs))
-	  (setq msgs (cdr msgs))
-	  (if (mew-smtp-insert-file pnm msg)
-	      (throw 'loop (setq flushp t)))))
-      (cons flushp msgs))))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Recovery
-;;;
-
-(defun mew-smtp-recover (pnm err)
-  (set-buffer (process-buffer (get-process pnm)))
-  (mew-smtp-log pnm err)
-  (mew-smtp-queue pnm err)
-  (let ((back (mew-smtp-get-messages pnm)))
-    (mew-smtp-close pnm)
-    (mew-smtp-set-messages pnm back)
-    (mew-smtp-send-next-message pnm 'no-msg))) ;; xxx
+     (mew-info-clean-up pnm mew-smtp-info-list-clean-length)
+     (set-process-buffer pro (current-buffer))
+     (mew-smtp-set-status pnm "mail-from")
+     (mew-smtp-command-mail-from pro pnm))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -645,7 +541,7 @@
       (and msgid (insert " id=" msgid))
       (and server (insert " server=" server))
       (and recipients
-	   (setq recipients (mapconcat (function identity) recipients ",")))
+	   (setq recipients (mapconcat 'identity recipients ",")))
       (and recipients (insert " recipients=" recipients))
       (if err
 	  (insert " status=" "("
