@@ -402,11 +402,15 @@ The keys that are defined for both Summary mode and Virtual mode are:
 \\[mew-summary-goto-folder]	Go to the folder which you input.
 
 \\[mew-summary-send]	Write a message. A new draft is prepared in Draft mode.
+
 \\[mew-summary-reply]	Answer to this message. A new draft is prepared
-	in Draft mode.  Mew automatically decides To: and Cc:.
+	in Draft mode. Mew automatically decides To: and Cc:.
+	When executed with '\\[universal-argument]', answer only to 
+	the sender.
 \\[mew-summary-reply-with-citation]	Answer to this message. A new
-	draft is prepared in Draft mode.  Mew automatically decides
-	To: and Cc: and cites the body.
+	draft is prepared in Draft mode. Mew automatically decides To:
+	and Cc: and cites the body.When executed with
+	'\\[universal-argument]', answer only to the sender.
 
 \\[mew-summary-forward]	Forward this message to a third person. A new
 	draft is prepared in Draft mode and this message is
@@ -504,10 +508,12 @@ The keys that are defined for both Summary mode and Virtual mode are:
 
 \\[mew-summary-burst]	De-capsulate embedded messages in MIME format.
 
-\\[mew-status-update]	Read Addrbook and update its information. If
-	executed with '\\[universal-argument]', information of folders
-	is also updated in addition to that of Addrbook. The default
-	value is 't'.
+\\[mew-status-update]	Read Addrbook and update its information. 
+	If executed with '\\[universal-argument]', information about local folders is 
+	also updated in addition to that of Addrbook. 
+	If executed with '1', information about local folders is updated.
+	If executed with '2', information about newsgroups is updated.
+	If executed with '3', information about IMAP folders is updated. 
 
 \\[mew-summary-set-case]	Set cases.
 
@@ -550,9 +556,20 @@ The keys that are defined for both Summary mode and Virtual mode are:
 
 \\[mew-summary-pipe-message]	Send this message via pipe.
 
-\\[mew-summary-send-message]	Flush messages stored in all queue folders.
-	If executed with '\\[universal-argument]' in a queue folder,
-	flush messages stored in the queue folder only.
+\\[mew-summary-send-message]	If in +queue, send the messages in +queue.
+	If in +postq, post the messages in +postq.
+	If in %queue, process the jobs in %queue.
+	Otherwise, flush the default queue.
+	If executed with '\\[universal-argument]', you can set the sending case.
+
+
+\\[mew-summary-toggle-8bit]	Toggle 8bit mode(i.e. 'mew-use-8bit').
+\\[mew-summary-cache-clean-up]	Clean-up caches of analized messages.
+\\[mew-summary-toggle-warning]	Toggle waring level.
+	If 'mew-warning-field-level' is 2, set it to 1.
+	If 'mew-warning-field-level' is 1, set it to 2.
+\\[mew-summary-toggle-policy]	Toggle decode policy(i.e. 'mew-decode-broken').
+\\[mew-summary-toggle-debug]	Toggle 'mew-debug'.
 
 The following commands are provided for Summary mode only, not for
 Virtual mode.
@@ -593,6 +610,16 @@ Virtual mode.
 \\[mew-summary-mark-unlink]	Put the 'X' mark onto all messages
 	marked with '*'.
 
+\\[mew-summary-exec-offline]	Process marked messages *offline*. Messages to be refiled in 
+	a remote folder are moved to the corresponding folder but they are
+	marked invalid. A number of an invalid message starts with '0'.
+	Invalid messages will be overridden when the remote folder
+	is scanned online. 
+	A job to delete or refile messages, which is created by this command, 
+	is queued in a queue folder (%queue for IMAP). To flush jobs in 
+	the queue, type '\\[mew-summary-send-message]' in the queue online.
+\\[mew-summary-local-copy]	Copying a cached message in a remote folder to a local folder.
+
 \\[mew-summary-search-mark]	Pick messages according to a pick pattern
 	which you input, then put the '*' mark onto them. If called
 	with '\\[universal-argument]', execute
