@@ -490,9 +490,11 @@ If called with '\\[universal-argument]', options are asked."
 ;;;
 
 (defun mew-summary-send-message (&optional arg)
-  "Flush messages stored in this queue folder or the default
-queue folder. If executed with '\\[universal-argument]',
-you can set the sending case."
+  "If in +queue, send the messages in +queue.
+If in +postq, post the messages in +postq.
+If in %queue, process the jobs in %queue.
+Otherwise, flush the default queue.
+If executed with '\\[universal-argument]', you can set the sending case."
   (interactive "P")
   (if (mew-folder-imap-queuep)
       (progn
@@ -800,31 +802,84 @@ to clean up."
 ;;;
 
 (defun mew-summary-local-copy ()
+  "Copying a cached message in a remote folder to a local folder."
   (interactive)
   (mew-summary-msg-or-part
    (let ((fld (mew-summary-folder-name))
-	 msg dstmsg dstfld file dstfile buf)
+	 (exclusivep t)
+	 msg dstmsg dstfld file dstfile)
      (save-excursion
-      (mew-summary-goto-message)
+       (mew-summary-goto-message)
        (setq msg (mew-summary-message-number)))
      (setq file (mew-expand-folder fld msg))
-     (setq dstfld (mew-input-folder mew-inbox-folder)) ;; xxx guess?
+     (setq dstfld (mew-input-folder mew-inbox-folder)) ;; xxx
      (setq dstmsg (mew-folder-new-message dstfld 'numonly))
      (setq dstfile (mew-expand-folder dstfld dstmsg))
-     (if (string= (file-name-directory file) (file-name-directory dstfile))
-	 (message "You cannot copy this to the same folder")
-       (setq buf (generate-new-buffer mew-buffer-prefix))
+     (when (get-buffer dstfld)
+       (set-buffer dstfld)
+       (setq exclusivep (mew-summary-exclusive-p)))
+     (cond
+      ((not exclusivep)
+       (message "Try again later"))
+      ((string= file dstfile)
+       (message "You cannot copy this onto itself"))
+      (t
+       (mew-summary-local-copy-one file dstfile)
+       (mew-touch-folder dstfld)
+       (message "Copied to %s/%s" dstfld dstmsg))))))
+
+(defun mew-summary-mark-local-copy (&optional arg)
+  "Copying marked cached message in a remote folder to a local folder."
+  (interactive "P")
+  (mew-summary-msg-or-part
+   (let ((mew-use-highlight-x-face nil)
+	 (fld (mew-summary-folder-name))
+	 (exclusivep t)
+	 msgs file dstmsg dstfld dstfile
+	 beg end region)
+     (cond
+      (arg
+       (setq region (mew-summary-get-region))
+       (setq beg (car region))
+       (setq end (cdr region)))
+      (t
+       (setq beg (point-min))
+       (setq end (point-max))))
+     (setq dstfld (mew-input-folder mew-inbox-folder)) ;; xxx
+     (setq dstmsg (mew-folder-new-message dstfld 'numonly))
+     (setq msgs (mew-summary-mark-collect mew-mark-review beg end))
+     (when (get-buffer dstfld)
        (save-excursion
-	 (set-buffer buf)
-	 (mew-erase-buffer)
-	 (mew-frwlet
-	  mew-cs-text-for-read mew-cs-text-for-write
-	  (insert-file-contents file)
-	  (goto-char (point-min))
-	  (mew-header-delete-lines (list mew-x-mew-uidl:))
-	  (write-region (point-min) (point-max) dstfile nil 'no-msg)))
-       (mew-remove-buffer buf)
-       (message "Copied to %s/%s" dstfld dstmsg)))))
+	 (set-buffer dstfld)
+	 (setq exclusivep (mew-summary-exclusive-p))))
+     (cond
+      ((null msgs)
+       (message "No mark"))
+      ((not exclusivep)
+       (message "Try again later"))
+      (t
+       (message "Copying...")
+       (while msgs
+	 (setq file (mew-expand-folder fld (car msgs)))
+	 (setq dstfile (mew-expand-folder dstfld dstmsg))
+	 (mew-summary-local-copy-one file dstfile)
+	 (setq dstmsg (number-to-string (1+ (string-to-number dstmsg))))
+	 (setq msgs (cdr msgs)))
+       (mew-touch-folder dstfld)
+       (message "Copying...done"))))))
+
+(defun mew-summary-local-copy-one (srcfile dstfile)
+  (let ((buf (generate-new-buffer mew-buffer-prefix)))
+    (save-excursion
+      (set-buffer buf)
+      (mew-erase-buffer)
+      (mew-frwlet
+       mew-cs-text-for-read mew-cs-text-for-write
+       (insert-file-contents srcfile)
+       (goto-char (point-min))
+       (mew-header-delete-lines (list mew-x-mew-uidl:))
+       (write-region (point-min) (point-max) dstfile nil 'no-msg)))
+    (mew-remove-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -865,20 +920,20 @@ you choose 'Summary mode only', you can quickly put the delete
 ;;;
 
 (defun mew-summary-toggle-8bit ()
+  "Toggle 8bit mode(i.e. 'mew-use-8bit')."
   (interactive)
   (setq mew-use-8bit (not mew-use-8bit))
   (if mew-use-8bit
       (message "mew-use-8bit has been set to 't'")
     (message "mew-use-8bit has been set to 'nil'")))
 
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Mew cache clean up
 ;;;
 
 (defun mew-summary-cache-clean-up ()
-  "Clean-up mew cache."
+  "Clean-up caches of analized messages."
   (interactive)
   (mew-cache-clean-up)
   (message "Mew cache clean up...done"))
@@ -889,7 +944,9 @@ you choose 'Summary mode only', you can quickly put the delete
 ;;;
 
 (defun mew-summary-toggle-warning ()
-  "Toggle waring level."
+  "Toggle waring level.
+If 'mew-warning-field-level' is 2, set it to 1.
+If 'mew-warning-field-level' is 1, set it to 2."
   (interactive)
   (if (= mew-warning-field-level 2)
       (setq mew-warning-field-level 1)
@@ -904,7 +961,7 @@ you choose 'Summary mode only', you can quickly put the delete
 ;;;
 
 (defun mew-summary-toggle-policy (&optional arg)
-  "Toggle decode policy."
+  "Toggle decode policy(i.e. 'mew-decode-broken')."
   (interactive "P")
   (if arg
       (setq mew-decode-broken nil
