@@ -182,17 +182,12 @@ All members must have the same length."
 			     ))
 
 (defsubst mew-thread-cache-valid-p (vfolder)
-  (let ((ofld (mew-virtual-to-folder vfolder)))
+  (let ((ofld (mew-thread-to-folder vfolder)))
     (when (get-buffer ofld)
       (save-excursion
 	(equal (progn (set-buffer ofld) (mew-sinfo-get-cache-time))
 	       (progn (set-buffer vfolder) (mew-sinfo-get-cache-time)))))))
 
-(defsubst mew-virtual-thread-p (vfolder)
-  (save-excursion  
-    (set-buffer vfolder)
-    (mew-thread-p)))
-
 (defun mew-summary-make-thread (&optional arg)
   "If called in Summary mode, this command makes threads for the
 Summary mode as Virtual mode, then the cursor jump onto the current
@@ -211,19 +206,21 @@ mode and move to the current message."
   (let* ((msg (mew-summary-message-number))
 	 (disp (mew-sinfo-get-disp-msg))
 	 (folder (mew-summary-folder-name 'ext)) ;; xxx
-	 vfolder)
+	 fld vfolder)
     (cond
      ((mew-virtual-p)
-      (setq folder (mew-virtual-to-folder folder))
-      (if (not (and folder (get-buffer folder)))
+      (if (not (mew-thread-p))
 	  (message "No physical folder")
-	(mew-summary-visit-folder folder nil 'no-ls)
-	(mew-summary-toggle-disp-msg (if disp 'on 'off))
-	(if (not msg)
-	    (goto-char (point-max))
-	  (mew-summary-jump-message msg)
-	  (mew-summary-display nil))))
-     ((and (setq vfolder (mew-folder-to-virtual folder))
+	(setq fld (mew-thread-to-folder folder))
+	(if (not (and fld (get-buffer fld)))
+	    (message "No physical folder")
+	  (mew-summary-visit-folder fld nil 'no-ls)
+	  (mew-summary-toggle-disp-msg (if disp 'on 'off))
+	  (if (not msg)
+	      (goto-char (point-max))
+	    (mew-summary-jump-message msg)
+	    (mew-summary-display nil)))))
+     ((and (setq vfolder (mew-folder-to-thread folder))
 	   (get-buffer vfolder)
 	   (mew-virtual-thread-p vfolder)
 	   (mew-thread-cache-valid-p vfolder))
@@ -246,7 +243,7 @@ threads are created, see 'mew-use-complete-thread'."
   (mew-summary-only
    (when (mew-summary-exclusive-p)
      (let* ((folder (mew-summary-folder-name 'ext))
-	    (vfolder (mew-folder-to-virtual folder))
+	    (vfolder (mew-folder-to-thread folder))
 	    (disp (mew-sinfo-get-disp-msg))
 	    (ctime (mew-sinfo-get-cache-time))
 	    (column (mew-summary-scan-form folder 'column))
