@@ -497,6 +497,7 @@ in draft buffer.")
     (,mew-keywords:      comma-text text)
     (,mew-received:      unstruct   unstruct)
     (,mew-references:    unstruct   unstruct)
+    (,mew-in-reply-to:   unstruct   unstruct)
     (,mew-x-face:	 unstruct   unstruct))
   "(field enc dec)")
 
@@ -869,7 +870,7 @@ following in .emacs.
 (defun mew-proto-to-body (fld n)
   (setq fld (mew-folder-folder fld))
   (if (mew-folder-virtualp fld)
-      (setq fld (mew-virtual-to-folder fld)))
+      (setq fld (mew-thread-to-folder fld)))
   (let ((proto (mew-folder-prefix fld)))
     (unless (member proto mew-folder-prefixes)
       (setq proto (mew-proto mew-case-input)))
