@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Oct 19, 2002
+;; Revised: Oct 24, 2002
 
 ;;; Commentary:
 
@@ -19,7 +19,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "3.0.70"
+(defconst mew-version-number "3.0.71"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -213,9 +213,12 @@ set mew-touch-folder-p to t"
 	      (set-file-modes dir mew-folder-mode))))))
 
 (defun mew-status-update (arg)
-  "Read Addrbook and update its information. If executed with '\\[universal-argument]',
-information of folders is also updated in addition to that of
-Addrbook. The default value is 't'."
+  "Read Addrbook and update its information. 
+If executed with '\\[universal-argument]', information about local folders is 
+also updated in addition to that of Addrbook. 
+If executed with '1', information about local folders is updated.
+If executed with '2', information about newsgroups is updated.
+If executed with '3', information about IMAP folders is updated. "
   (interactive "P")
   (message "Updating status...")
   (cond
